///////////////////////////////////////////////////////////////////////
//
// TAMthLnk.cpp : techexplorer MthLnk AddIn "app" implementation file
//
///////////////////////////////////////////////////////////////////////
//
// Copyright (c) 1997-98 by the IBM Corporation. All Rights Reserved
//
// Author:          Angel L. Diaz     aldiaz@us.ibm.com
//                  Robert S. Sutor   sutor@us.ibm.com
//
// IBM MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
// TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
// PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR
// ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING OR MODIFYING
// THIS SOFTWARE OR ITS DERIVATIVES.
//
///////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "TAMthLnk.h"
#include "TADialog.h"
#include "mathlink.h"

#ifdef _DEBUG
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif

int    inargc   = 4;
char  *inargv[] = {"-linkname", "math -mathlink", "-linkmode", "launch"};

// =====================================================================
//
// CTAMthLnkApp: Constructor/Destructor
//
// =====================================================================

CTAMthLnkApp::CTAMthLnkApp() :
    f_sMimeType( _T( "application/x-mathlink-string;.huh;Mathematica MathLink Evaluation Strings" ) )
{
}

CTAMthLnkApp theApp; // The one and only CTAMthLnkApp object

CTAMthLnkApp*
CTAMthLnkApp::GetAddInDllObject()
{
    return &theApp;
}

// =====================================================================
//
// CTAMthLnkApp: Add-In/Instance operations
//
// =====================================================================

TAError
CTAMthLnkApp::Initialize()
{
   long err;

   TRACE("CTAMthLnkApp::Initialize()\n");
   ep =  MLInitialize( (MLParametersPointer)0);
   if( ep == (MLENV)0) 
       {
       MLDeinitialize( ep);
       exit(1);
       }
   lp = MLOpenArgv( ep, inargv, inargv + inargc, &err);
   if(lp == (MLINK)0) 
       {
       MLDeinitialize( ep);
       MLClose( lp);
       exit(2);
       }
   return TAERROR_OK;
}

TAError
CTAMthLnkApp::Shutdown()
{
   TRACE("CTAMthLnkApp::Shutdown(): Number of instances %d\n",f_Instances.GetCount());
   if( lp) MLClose( lp);
   if( ep) MLDeinitialize( ep);
   return TAERROR_OK;
}

TAError
CTAMthLnkApp::New(AddInInstance *pAddInInstance, LPCSTR sMimeType)
{
    TRACE("CTAMthLnkApp::New(): instance = %p : Number of instances %d\n",pAddInInstance, f_Instances.GetCount());

    if ( pAddInInstance == NULL )
        return TAERROR_INSTANCE;
 
    f_Instances.AddHead( pAddInInstance );

   return TAERROR_OK;
}

TAError
CTAMthLnkApp::Destroy(AddInInstance *pAddInInstance)
{
   TRACE("CTAMthLnkApp::Destroy(): instance = %p : Number of instances %d\n",pAddInInstance, f_Instances.GetCount());
   if ( pAddInInstance == NULL )
       return TAERROR_INSTANCE;
 
   POSITION pos( f_Instances.Find( pAddInInstance ) );

   if ( pos != NULL )
      f_Instances.RemoveAt( pos );

   return TAERROR_OK;
}

// =====================================================================
//
// CTAMthLnkApp: Evaluation API
//
// =====================================================================

void error( MLINK lp)
{
	if (MLError( lp)) {
		fprintf( stderr, "Error detected by MathLink: %s.\n",
		MLErrorMessage( lp));
	}else{
		fprintf( stderr, "Error detected by this program.\n");
	}
	exit( 1);
}

TAError
CTAMthLnkApp::Evaluate(AddInInstance *pAddInInstance, LPCSTR sMimeType, LPCSTR sAppInput)
{
    int         pkt;         
    const char *strout;
    TAError     nResult = TAERROR_OK;

    TRACE("CTAMthLnkApp::Evaluate(): sMimeType %s\n",LPCSTR( sMimeType ));
    TRACE("CTAMthLnkApp::Evaluate(): sAppInput %s\n",LPCSTR( sAppInput ));

    if ( sMimeType && sAppInput ) 
    {
    if( ::strcmp("application/x-mathlink-string", sMimeType ) == 0 ) 
        {
            MLPutFunction(lp, "ToString", 1L);
            MLPutFunction(lp, "ToExpression", 1L);
            MLPutByteString(lp, (ucharp_ct)sAppInput, strlen(sAppInput));
	        MLEndPacket(lp);

            /* skip any packets before the first ReturnPacket */
	        while( (pkt = MLNextPacket( lp), pkt) && pkt != RETURNPKT) 
            {
	            MLNewPacket( lp);
    	        if( MLError( lp)) error( lp);
            }
            MLGetString(lp, &strout);

            CRequestedFileDialog RequestedFileDialog( strout );
		    RequestedFileDialog.DoModal();
            MLDisownString(lp, strout);
        }
		else
	        nResult = TAERROR_FAIL;
    }
    else
        nResult = TAERROR_FAIL; 

   return nResult;
}

// =====================================================================
//
// CTAMthLnkApp: Asynchronous operations
//
// =====================================================================

TAError                      
CTAMthLnkApp::LocalFileIsReady(AddInInstance *pAddInInstance, LPCSTR sFileName)
{
   TAError nResult = TAERROR_OK;
   CString sUrlContents = _T("");

   if ( sFileName != NULL ) {
	   TRACE("CTAMthLnkApp::LocalFileIsReady(): sFileName %s\n",LPCSTR( sFileName ));
	   const UINT nBufferSize = 1024;
	   char buffer[ nBufferSize ];

	   CStdioFile file;
	   
	   if ( file.Open( sFileName, CFile::modeRead | CFile::shareDenyWrite ) ) {
		   UINT nBytesRead = file.Read( buffer, nBufferSize-1 );
		   
		   while ( nBytesRead > 0 ) {
			   buffer[ nBytesRead ] = '\0';
               sUrlContents += buffer;
               nBytesRead = file.Read( buffer, nBufferSize-1 );
		   }
		   file.Close();
		   
		   CleanUpWhiteSpace( sUrlContents  );
		   CRequestedFileDialog RequestedFileDialog( sUrlContents );
		   RequestedFileDialog.DoModal();
	   }
	   else
		   nResult = TAERROR_FAIL;
   }
   else
	   nResult = TAERROR_FAIL;

   // activate the \evalLink
   techexplorerEnableEvaluate( pAddInInstance );

   return TAERROR_OK;
}

// =====================================================================
//
// CTAMthLnkApp: Utillity
//
// =====================================================================

LPCSTR
CTAMthLnkApp::GetMimeType()
{
   TRACE("CTAMthLnkApp::GetMimeType(): %s\n",LPCSTR( f_sMimeType ));
   return f_sMimeType;
}

void
CTAMthLnkApp::CleanUpWhiteSpace( CString &rInput )
{
    int nLength = rInput.GetLength();

    CString sBuffer = rInput;
    rInput.Empty();

    for ( int i = 0; i < nLength; i++ ) {
        TCHAR c = sBuffer[i];
        
        if ( c == '\n' ) 
            rInput += "\r\n";
        else
           if ( c != '\r' )
              rInput += c;
    }
}
